#include "StdAfx.h"
#include "CGO_GeoObject.h"
#include "CLogger.h"

CGO_Circle::CGO_Circle(void) {
	m_ObjectId = CGO_GeoObject::idFactory;
	m_DisplayListId = m_ObjectId + LISTID_OFFSET;

	char message [200];
	sprintf(message, "New circle created: object id #%i", m_ObjectId);
	CLogger::getInstance()->info(message);

	CGO_GeoObject::idFactory++;
}

CGO_Circle::~CGO_Circle(void) {
}

IMPLEMENT_SERIAL(CGO_Circle, CGO_GeoObject, 0)
void CGO_Circle::Serialize(CArchive &a) {
	char buffer [200];

	if ( a.IsStoring() ) {
		sprintf(buffer, "Serializing circle #%i (x: %4.1f, y: %4.1f, r: %4.1f)...", m_ObjectId, m_Center.getX(), m_Center.getY(), m_Radius );
		CLogger::getInstance()->debug(buffer);

		a << m_Center.getX() << m_Center.getY() << m_Radius;
	} else {
		float x, y;
		a >> x >> y >> m_Radius;
		m_Center.setCoords(x, y);

		sprintf(buffer, "Circle unserialized. (id: %i, x: %4.1f, y: %4.1f, r: %4.1f)", m_ObjectId, x, y, m_Radius );
		CLogger::getInstance()->debug(buffer);
	}
}

void CGO_Circle::removeObject() {
	glDeleteLists(m_DisplayListId, 1);

	char message [200];
	sprintf(message, "Display list for circle #%i deleted.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::updateRadius() {
	m_Radius = (float) sqrt( pow(m_Peripheral.getX() - m_Center.getX(), 2) + pow(m_Peripheral.getY() - m_Center.getY(), 2) );

	char message [200];
	sprintf(message, "Radius of circle #%i updated: %4.1f", m_ObjectId, m_Radius);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::setByPeripheral(CGO_Point c, CGO_Point p) {
	m_Center = c;
	m_Peripheral = p;

	char message [200];
	sprintf(message, "Circle #%i: C(%4.1f, %4.1f), P(%4.1f, %4.1f)", m_ObjectId, m_Center.getX(), m_Center.getY(), m_Peripheral.getX(), m_Peripheral.getY());
	CLogger::getInstance()->info(message);

	this->updateRadius();
}

void CGO_Circle::setByRadius(CGO_Point c, float radius) {
	m_Radius = radius;
	m_Center = c;
	m_Peripheral.setCoords( m_Center.getX(), m_Center.getY() + radius );

	char message [200];
	sprintf(message, "Circle #%i: C(%4.1f, %4.1f), P(%4.1f, %4.1f), radius: %4.1f", m_ObjectId, m_Center.getX(), m_Center.getY(), m_Peripheral.getX(), m_Peripheral.getY(), m_Radius);
	CLogger::getInstance()->info(message);
}

float CGO_Circle::getRadius() {
	return m_Radius; 
}

void CGO_Circle::draw() {
	float rad;

	if ( glIsList(m_DisplayListId) ) {
		glDeleteLists(m_DisplayListId, 1);
	}
	glNewList(m_DisplayListId, GL_COMPILE);
		glBegin(GL_LINE_STRIP);
		for (int i = 0; i <= 360; i++) {
			rad = (i * 3.14159265f) / 180;
			glVertex2f( (float) cos(rad) * m_Radius + m_Center.getX() , (float) sin(rad) * getRadius() + m_Center.getY() );
		}
		glEnd();
	glEndList();

	char message [200];
	sprintf(message, "Display list #%i for circle #%i generated.", m_DisplayListId, m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::move(CGO_Vector* v) {
	m_Center.move(v);
	m_Peripheral.move(v);

	char message [200];
	sprintf(message, "Circle #%i moved to C(%4.1f, %4.1f), P(%4.1f, %4.1f).", m_ObjectId, m_Center.getX(), m_Center.getY(), m_Peripheral.getX(), m_Peripheral.getY());
	CLogger::getInstance()->info(message);
}

void CGO_Circle::rotate(float angle) {
	m_Center.rotate(angle);
	m_Peripheral.rotate(angle);

	char message [200];
	sprintf(message, "Circle #%i rotated by %4.1f degree.", m_ObjectId, angle);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::rotateBasepoint(CGO_Point p, float angle) {
	m_Center.rotateBasepoint(p, angle);
	m_Peripheral.rotateBasepoint(p, angle);

	char message [200];
	sprintf(message, "Circle #%i rotated by %4.1f degree.", m_ObjectId, angle);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::scale(float factor) {
	m_Center.scale(factor);
	m_Peripheral.scale(factor);
	this->updateRadius();

	char message [200];
	sprintf(message, "Circle #%i scaled by %4.1f (new radius: %4.1f).", m_ObjectId, factor, m_Radius);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::scaleBasepoint(CGO_Point p, float factor) {
	m_Center.scaleBasepoint(p, factor);
	m_Peripheral.scaleBasepoint(p, factor);
	this->updateRadius();

	char message [200];
	sprintf(message, "Circle #%i scaled by %4.1f (new radius: %4.1f).", m_ObjectId, factor, m_Radius);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::flipX() {
	m_Center.flipX();
	m_Peripheral.flipX();

	char message [200];
	sprintf(message, "Circle #%i flipped vertical.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::flipY() {
	m_Center.flipY();
	m_Peripheral.flipY();

	char message [200];
	sprintf(message, "Circle #%i flipped horizontal.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::flipOrigin() {
	m_Center.flipOrigin();
	m_Peripheral.flipOrigin();

	char message [200];
	sprintf(message, "Circle #%i flipped on origin of ordinates.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Circle::flipStraight(CGO_Point p1, CGO_Point p2) {
	m_Center.flipStraight(p1, p2);
	m_Peripheral.flipStraight(p1, p2);

	char message [200];
	sprintf(message, "Circle #%i flipped on straight through P1(%4.1f, %4.1f), P2(%4.1f, %4.1f).", m_ObjectId, p1.getX(), p1.getY(), p2.getX(), p2.getY());
	CLogger::getInstance()->info(message);
}