#include "StdAfx.h"
#include "CGO_GeoObject.h"
#include "CLogger.h"

CGO_Line::CGO_Line() {
	m_ObjectId = CGO_GeoObject::idFactory;
	m_DisplayListId = m_ObjectId + LISTID_OFFSET;

	char message [200];
	sprintf(message, "New line created: object id #%i", m_ObjectId);
	CLogger::getInstance()->info(message);

	CGO_GeoObject::idFactory++;
}

CGO_Line::~CGO_Line() {
}

IMPLEMENT_SERIAL(CGO_Line, CGO_GeoObject, 0)
void CGO_Line::Serialize(CArchive &a) {
	char buffer [200];

	if ( a.IsStoring() ) {
		sprintf(buffer, "Serializing line #%i (x1: %4.1f, y1: %4.1f, x2: %4.1f, y2: %4.1f)...", m_ObjectId, m_P1.getX(), m_P1.getY(), m_P2.getX(), m_P2.getY() );
		CLogger::getInstance()->debug(buffer);

		a << m_P1.getX() << m_P1.getY() << m_P2.getX() << m_P2.getY();
	} else {
		float x1, y1, x2, y2;
		a >> x1 >> y1 >> x2 >> y2;
		m_P1.setCoords(x1, y1);
		m_P2.setCoords(x2, y2);

		sprintf(buffer, "Line unserialized. (id: %i, x1: %4.1f, y1: %4.1f, x2: %4.1f, y2: %4.1f)", m_ObjectId, x1, y1, x2, y2 );
		CLogger::getInstance()->debug(buffer);
	}
}
	
void CGO_Line::removeObject() {
	glDeleteLists(m_DisplayListId, 1);

	char message [200];
	sprintf(message, "Display list for line #%i deleted.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Line::setPoints(CGO_Point p1, CGO_Point p2) {
	m_P1 = p1;
	m_P2 = p2;
	
	char message [200];
	sprintf(message, "Line #%i: P1(%4.1f, %4.1f), P2(%4.1f, %4.1f)", m_ObjectId, m_P1.getX(), m_P1.getY(), m_P2.getX(), m_P2.getY());
	CLogger::getInstance()->info(message);
}


void CGO_Line::draw() {
	if ( glIsList(m_DisplayListId) ) {
		glDeleteLists(m_DisplayListId, 1);
	}
	glNewList(m_DisplayListId, GL_COMPILE);
		glBegin(GL_LINES);
			glVertex2f( m_P1.getX() , m_P1.getY() );
			glVertex2f( m_P2.getX() , m_P2.getY() );
		glEnd();
	glEndList();

	char message [200];
	sprintf(message, "Display list #%i for line #%i generated.", m_DisplayListId, m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Line::move(CGO_Vector* v) {
	m_P1.move(v);
	m_P2.move(v);

	char message [200];
	sprintf(message, "Line #%i moved to P1(%4.1f, %4.1f), P2(%4.1f, %4.1f).", m_ObjectId, m_P1.getX(), m_P1.getY(), m_P2.getX(), m_P2.getY());
	CLogger::getInstance()->info(message);
}


void CGO_Line::rotate(float angle) {
	m_P1.rotate(angle);
	m_P2.rotate(angle);

	char message [200];
	sprintf(message, "Line #%i rotated by %4.1f degree.", m_ObjectId, angle);
	CLogger::getInstance()->info(message);
}

void CGO_Line::rotateBasepoint(CGO_Point p, float angle) {
	m_P1.rotateBasepoint(p, angle);
	m_P2.rotateBasepoint(p, angle);

	char message [200];
	sprintf(message, "Line #%i rotated by %4.1f degree.", m_ObjectId, angle);
	CLogger::getInstance()->info(message);
}

void CGO_Line::scale(float factor) {
	m_P1.scale(factor);
	m_P2.scale(factor);

	char message [200];
	sprintf(message, "Line #%i scaled by %4.1f.", m_ObjectId, factor);
	CLogger::getInstance()->info(message);
}

void CGO_Line::scaleBasepoint(CGO_Point p, float factor) {
	m_P1.scaleBasepoint(p, factor);
	m_P2.scaleBasepoint(p, factor);

	char message [200];
	sprintf(message, "Line #%i scaled by %4.1f.", m_ObjectId, factor);
	CLogger::getInstance()->info(message);
}

void CGO_Line::flipX() {
	m_P1.flipX();
	m_P2.flipX();

	char message [200];
	sprintf(message, "Line #%i flipped horizontal.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Line::flipY() {
	m_P1.flipY();
	m_P2.flipY();

	char message [200];
	sprintf(message, "Line #%i flipped vertical.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Line::flipOrigin() {
	m_P1.flipOrigin();
	m_P2.flipOrigin();

	char message [200];
	sprintf(message, "Line #%i flipped on origin of ordinates.", m_ObjectId);
	CLogger::getInstance()->info(message);
}

void CGO_Line::flipStraight(CGO_Point p1, CGO_Point p2) {
	m_P1.flipStraight(p1, p2);
	m_P2.flipStraight(p1, p2);

	char message [200];
	sprintf(message, "Line #%i flipped on straight through P1(%4.1f, %4.1f), P2(%4.1f, %4.1f).", m_ObjectId, p1.getX(), p1.getY(), p2.getX(), p2.getY());
	CLogger::getInstance()->info(message);
}