#include "StdAfx.h"

CGO_Point::CGO_Point(void) {
	m_X = 0;
	m_Y = 0;
}

CGO_Point::~CGO_Point(void) {
}

void CGO_Point::setCoords(float x, float y) {
	m_X = x;
	m_Y = y;
}

void CGO_Point::setX(float x) {
	m_X = x;
}

void CGO_Point::setY(float y) {
	m_Y = y;
}

float CGO_Point::getX() {
	return m_X;
}

float CGO_Point::getY() {
	return m_Y;
}

void CGO_Point::move(CGO_Vector* v) {
	m_X = m_X + v->getDX();
	m_Y = m_Y + v->getDY();
	
	char message  [200];
	sprintf(message, "Point moved. New location: (%4.1f, %4.1f)", m_X, m_Y);
	CLogger::getInstance()->debug(message);
}

void CGO_Point::rotate(float angle) {
	float rad = angle * (float) M_PI / 180;

	float x_new = m_X * cos(rad) - m_Y * sin(rad);
	float y_new = m_Y * cos(rad) + m_X * sin(rad);
	m_X = x_new;
	m_Y = y_new;

	char message  [200];
	sprintf(message, "Point rotated by %4.1f degree (%5.3f rad). New location: (%4.1f, %4.1f).", angle, rad, m_X, m_Y);
	CLogger::getInstance()->debug(message);
}

void CGO_Point::rotateBasepoint(CGO_Point p, float angle) {
	char message [200];
	CGO_Vector* pVector = new CGO_Vector();

	//move basepoint to origin of ordinates
	pVector->setCoords( -p.getX(), -p.getY());
	this->move(pVector);

	sprintf(message, "Rotating: setting basepoint B(%4.1f, %4.1f).",  p.getX(), p.getY());
	CLogger::getInstance()->debug(message);

	//rotate
	this->rotate(angle);

	//undo moving
	pVector->setCoords( p.getX(), p.getY() );
	this->move(pVector);

	CLogger::getInstance()->debug("Rotating: unsetting basepoint.");
}

void CGO_Point::scale(float factor) {
	m_X = m_X * factor;
	m_Y = m_Y * factor;

	char message [200];
	sprintf(message, "Point scaled by factor %4.1f. New location: (%4.1f, %4.1f).", factor, m_X, m_Y);
	CLogger::getInstance()->debug(message);
}

void CGO_Point::scaleBasepoint(CGO_Point p, float factor) {
	char message [200];
	CGO_Vector* pVector = new CGO_Vector();

	//move basepoint to origin of ordinates
	pVector->setCoords( -p.getX(), -p.getY() );
	this->move(pVector);

	sprintf(message, "Scaling: setting basepoint B(%4.1f, %4.1f).", p.getX(), p.getY());
	CLogger::getInstance()->debug(message);

	//scale
	this->scale(factor);

	//undo moving
	pVector->setCoords( p.getX(), p.getY() );
	this->move(pVector);

	CLogger::getInstance()->debug("Scaling: unsetting basepoint.");
}

void CGO_Point::flipX() {
	m_Y = -m_Y;

	char message  [200];
	sprintf(message, "Point flipped horizontal. New location: (%4.1f, %4.1f)", m_X, m_Y);
	CLogger::getInstance()->debug(message);
}

void CGO_Point::flipY() {
	m_X = -m_X;

	char message  [200];
	sprintf(message, "Point flipped vertical. New location: (%4.1f, %4.1f)", m_X, m_Y);
	CLogger::getInstance()->debug(message);
}

void CGO_Point::flipOrigin() {
	this->flipX();
	this->flipY();
}

void CGO_Point::flipStraight(CGO_Point p1, CGO_Point p2) {
	/**
	 * 2) Drehung um P1 um -angle Grad - Spiegelgerade parallel zur x-Achse
	 * 1) Spiegelgerade auf x-Achse verschieben
	 * 3) erhaltenen Punkt an der x-Achse spiegeln
	 * 4) Spiegelgerade zurckschieben
	 * 5) und ursprnglichen Winkel wieder herstellen.
	 */
	CGO_Vector* v = new CGO_Vector();
	float angle;

	v->setPoints(p1, p2);
	angle = v->getAngle();

	this->rotateBasepoint(p1, -angle);
	v->setCoords( 0, -p1.getY() );
	this->move(v);
	this->flipX();
	v->setCoords( 0, p1.getY() );
	this->move(v);
	this->rotateBasepoint(p1, angle);

	char message  [200];
	sprintf(message, "Point flipped on straight through P1(%4.1f, %4.1f), P2(%4.1f, %4.1f). New location: (%4.1f, %4.1f).", p1.getX(), p1.getY(), p2.getX(), p2.getY(), m_X, m_Y);
	CLogger::getInstance()->debug(message);
}