#include "StdAfx.h"
#include "CLogger.h"

CDatabase::CDatabase() {
	m_iCount = 0;
}

CDatabase::~CDatabase() {
}

void CDatabase::save(LPCTSTR fileName) {
	CFile file;
	file.Open(fileName, CFile::modeWrite | CFile::modeCreate);
	CArchive a(&file, CArchive::store);
	m_Database.Serialize(a);
}

int CDatabase::load(LPCTSTR fileName) {
	char buffer [200];

	this->removeAll();

	sprintf(buffer, "Loading graphics from file '%s'...", fileName);
	CLogger::getInstance()->info(buffer);
	
	CFile file;
	file.Open(fileName, CFile::modeRead);
	CArchive a(&file, CArchive::load);
	m_Database.Serialize(a);
	m_iCount = (int) m_Database.GetCount();

	sprintf(buffer, "%i graphics loaded. Drawing...", m_iCount);
	CLogger::getInstance()->info(buffer);

	this->drawObjects();
	return this->getCount();
}

void CDatabase::addObject(CGO_GeoObject* pObject) {
	pObject->draw();
	int pos = (int) m_Database.Add( pObject );
	m_iCount++;
	
	char message [200];
	sprintf(message, "Object #%i added to database at position %i.", pObject->getObjectId(), pos);
	CLogger::getInstance()->debug(message);
}

void CDatabase::removeObject(CGO_GeoObject* pObject) {
	if ( pObject == NULL ) {
		return;
	}
	
	CGO_GeoObject* pObj;
	char message [200];

	for ( int i = 0; i < m_Database.GetCount(); i++ ) {
		pObj = (CGO_GeoObject*) m_Database[i];

		if ( pObject != pObj ) {
			continue;
		}

		sprintf(message, "Removing object #%i from database...", pObject->getObjectId());
		CLogger::getInstance()->debug(message);

		m_Database[i] = NULL;
		m_iCount--;
		pObject->removeObject();
		delete pObject;
		return;
	}
}

void CDatabase::removeObjectById(int oId) {
	this->removeObject( this->getObject(oId) );
}

void CDatabase::removeAll() {
	CGO_GeoObject* pObject;

	CLogger::getInstance()->debug("Clearing Database...");

	for ( int i = 0; i < m_Database.GetCount(); i++ ) {
		pObject = (CGO_GeoObject*) m_Database[i];
		if ( pObject == NULL ) {
			continue;
		}
		this->removeObject(pObject);
	}

	CLogger::getInstance()->debug("Database cleared.");

	CGO_GeoObject::resetIdFactory();
}

void CDatabase::drawObjects() {
	CGO_GeoObject* pObject;
	
	char message [200];
	sprintf(message, "Drawing %i objects in database.", m_iCount);
	CLogger::getInstance()->debug(message);

	for ( int i = 0; i < m_Database.GetCount(); i++ ) {
		pObject = (CGO_GeoObject*) m_Database[i];
		if (pObject == NULL) {
			continue;
		}
		pObject->draw();
	}
}

CGO_GeoObject* CDatabase::getObject(int oId) {
	CGO_GeoObject* pObject;
	char message [200];

	if ( oId < 0 ) {
		sprintf(message, "Illegal object id #%i!", oId);
		CLogger::getInstance()->warning(message);
        return NULL;
	}

	for ( int i = 0; i < m_Database.GetCount(); i++ ) {
		pObject = (CGO_GeoObject*) m_Database[i];

		if (pObject == NULL) {
			continue;
		}

		if (pObject->getObjectId() == oId) {
			sprintf(message, "Object #%i found in database.", oId);
			CLogger::getInstance()->debug(message);
			
			return pObject;
		}
	}

	sprintf(message, "Object #%i was not found in database.", oId);
	CLogger::getInstance()->warning(message);
	
	return NULL;
}

int CDatabase::getCount() {
	return (int) m_Database.GetCount();
}

int CDatabase::getNumberObjects() {
	return m_iCount;
}

int CDatabase::getNextId() {
	return (int) m_Database.GetCount();
}